/********************************************************************
*                   LIBRERIA USO USART
* 
*   - Funciones para transmision y recepcion de caracteres
 *******************************************************************/

#ifndef USART_H
#define USART_H

#include <stdbool.h>
#include "config.h"
#include <stdint.h>

#define GetSystemClock() _XTAL_FREQ     //Indica la frecuencia del micro para calcular el baud rate

#define UART_TRISTx   TRISCbits.TRISC6  //Indica los pines de RX y TX
#define UART_TRISRx   TRISCbits.TRISC7
#define UART_Tx       PORTCbits.RC6
#define UART_Rx       PORTCbits.RC7
#define BAUD          19200             //Indica la frecuencia para la comunicacion
#define MULTIPLIER 16UL                 //Indica el multiplicador 64UL, 16UL or 4UL
#define HIGH_SPEED 1                    //Indica el BRGH. Segun la velocidad necesitamos h o l. Ver datasheet



//The two registers that control the USART are RCSTA and TXSTA, which aremnemonics 
//for "receive status" and "transmit status".
    
//Setting SPEN =1 enables the serial port, setting CREN = 1 enables (continuous) reception, 
//and setting TXEN =1enables transmitting.
    
//RCSTA = 0b10010000; // 0x90(SPEN RX9 SREN CREN ADEN FERR OERR RX9D)
//TXSTA = 0b00100000; // 0x20 (CSRC TX9 TXEN SYNC - BRGH TRMT TX9D)

#define UART_ENABLE   RCSTAbits.SPEN
#define UART_RECEP_ENABLE   RCSTAbits.CREN
#define UART_TRANS_ENABLE   TXSTAbits.TXEN
#define UART_RCSTA RCSTA1
#define UART_TXSTA TXSTA1
#define RCSTA_DEFAULT 0x90
#define TXSTA_DEFAULT 0x20


//Baud rate generator
//To set the baud rate, you will need to set the SPBRG register to a "divider" value 
    
//Low speed (BRGH=0): Baud Rate = Fosc/ (64(X+1))
//High speed (BRGH=1): Baud Rate = Fosc/ (16(X+1))
#define SPBRG   SPBRG1


#define RCIF          RC1IF     //Bandera que indica recepcion
#define TXIF          TX1IF     //Bandera que indica registro vacio



/******************************************************************************
*                       CONFIGURACION DEL PUERTO
* 
*   - Realiza la inicializacin del puerto segun los valores anteriores
 *****************************************************************************/
void USART_Initialize();

/******************************************************************************
*                       ESCRIBIR CARACTER
* 
*   - Funcion que transmite un caracter cuando el buffer de salida esta libre
*   - La funcin "printf" llama por defecto a la funcion putch  
 *****************************************************************************/
void USART_putcUSART(char c);
void putch (char c);


/******************************************************************************
*                       ESTADO BUFFER SALIDA
* 
*   -  Funcin que devuelve el estado del buffer de salida
*      TRUE: Libre,  FALSE: Ocupado 
 *****************************************************************************/
bool UART_TX_Empty(void);


/******************************************************************************
*                       ESCRIBIR FRASE
* 
*   - Funcion encargada de escribir caracteres contenidos en un array usando
*     la funcin USART_putcUSART 
 *****************************************************************************/
void UART_Write_Text(char *text);


/******************************************************************************
*                       RECIBIDO ALGO
* 
*   - Funcion que devuelve el estado del buffer de entrada 
*     TRUE: Libre,  FALSE: Ocupado 
 *****************************************************************************/
bool UART_Data_Ready(void);


/******************************************************************************
*                       LEER CARACTER
* 
*   - Funcion que devuelve el caracter leido del buffer de entrada 
 *****************************************************************************/
unsigned char USART_getcUSART(void);


/******************************************************************************
*                       LEER CARACTER O ESPERAR CARACTER
* 
*   - Funcion que devuelve el caracter leido del buffer de entrada 
*   - Esta funcion se diferencia de la anterior que espera si no hay nada en
*     el buffer. Cuando lo reciba lo devuelve.  
 *****************************************************************************/
unsigned char USART_getcUSART2 (void);


/******************************************************************************
*                       LEER GRUPO DE CARACTERES
* 
*   - Funcion que guarda en un array el contenido del buffer de entrada
*   - Usa la funcion que espera por caracter en buffer y no termina hasta que 
*     haya completado el array. 
 *****************************************************************************/
void USART_ReadString(char *Output, unsigned int length);


/******************************************************************************
*                       LEER GRUPO DE CARACTERES
* 
*   - Funcion que guarda en un array el contenido del buffer de entrada
*   - Usa la funcion que espera por caracter en buffer y no termina hasta que 
*     encuentre el caracter fin de linea '\n'
 *****************************************************************************/
void USART_ReadString2(char *Output);



#endif //USART_H
